% Max Likelihood Estimation of P matrix using EM algorithm

% Inputs: iter_max is the number of iterations, P is the initial estimate
% for the 2 x 2 P matrix, and Y is a row vector representing a subsampled
% 2-state Markov chain, where 1s and 2s are states and 0s are "blanks".

function P = EM(iter_max,P,Y)

    if P(1) == 0
       P(1) = 10^-10;
    end
    if P(2) == 0
       P(2) = 10^-10;
    end
    if P(3) == 0
       P(3) = 10^-10;
    end
    if P(4) == 0
       P(4) = 10^-10;
    end

    % Calculate the Omnt's
    [m,n,t] = Omnt(Y);

    numStates = 2;
    O(:,:,:) = zeros(numStates,numStates,length(Y)-1);
    F = zeros(length(m),1);
    for x = 1:length(m) 
        if (F(x) == 0)
            for xx = x:length(m)
                if ((F(xx) == 0) && (m(x) == m(xx)) && (n(x) == n(xx)) && (t(x) == t(xx)))
                    F(xx) = 1;
                    O(m(x),n(x),t(x)) = O(m(x),n(x),t(x)) + 1;
                end
            end
        end
    end

    % EM algorithm
    S_sum = zeros(numStates^2,1);
    for iter = 1:iter_max
        % E-step
        index = 0;
        for i = 1:numStates
            for j = 1:numStates
                index = index + 1;
                for M = 1:max(m)
                    for N = 1:max(n)
                         for T = 1:max(t)
                             for K = 0:T-1
                                P_K = P^K;
                                P_TK1 = P^(T-K-1);
                                P_T = P^T;
                                S = O(M,N,T)*(P_K(M,i)*P(i,j)*P_TK1(j,N)/P_T(M,N));
                                S_sum(index) = S + S_sum(index);
                            end
                         end
                    end
                end
                if i == 1
                    if j == 1
                        S_11 = S_sum(1);
                    else
                        S_12 = S_sum(2);
                    end
                elseif i == 2
                    if j == 1
                        S_21 = S_sum(3);
                    else
                        S_22 = S_sum(4);
                    end
                end
            end
        end

        % M-step
        P_11 = S_11/(S_11 + S_12);
        if P_11 == 0
            P_11 = 10^-10;
        end
        P_12 = S_12/(S_11 + S_12);
        if P_12 == 0
            P_12 = 10^-10;
        end
        P_21 = S_21/(S_21 + S_22);
        if P_21 == 0
            P_21 = 10^-10;
        end
        P_22 = S_22/(S_21 + S_22);
        if P_22 == 0
            P_22 = 10^-10;
        end

        P = [P_11 P_12;P_21 P_22];

    end
end