% This function inputs a Y vector of 1s, 2s, and 0s, where 1s and 2s are
% states, and 0s are "blanks" in a 2-state Markov chain. It returns m, n,
% and t vectors, which define the Markov chain in terms of all combinations
% of going from state m to state n in t time slots.

function [m,n,t] = Omnt(Y)

    A = find(Y ~= 0);
    B = Y(A);

    t = zeros((length(A)-1),1);
    for a = 1:length(A)-1
        t(a) = A(a+1)-A(a);
    end

    m = zeros((length(B)-1),1);
    n = zeros((length(B)-1),1);
    for b = 1:length(B)-1
        if B(b+1) - B(b) == -1
            m(b) = 2;
            n(b) = 1;
        end
        if B(b+1) - B(b) == 1
            m(b) = 1;
            n(b) = 2;
        end
        if B(b+1) - B(b) == 0
            if B(b+1) == 1
                m(b) = 1;
                n(b) = 1;
            end
            if B(b+1) == 2
                m(b) = 2;
                n(b) = 2;
            end
        end
    end
    
end