% Max Likelihood Estimation of P matrix using Exhaustive Search

% Input: Y is a row vector representing a subsampled 2-state Markov chain, 
% where 1s and 2s are states and 0s are "blanks".

function P = exhaust(Y)

    % Calculate the Oijt's
    [i,j,t] = Omnt(Y);

    numStates = 2;
    O(:,:,:) = zeros(numStates,numStates,length(Y)-1);
    F = zeros(length(i),1);
    for x = 1:length(i) 
        if (F(x) == 0)
            for xx = x:length(i)
                if ((F(xx) == 0) && (i(x) == i(xx)) && (j(x) == j(xx)) && (t(x) == t(xx)))
                    F(xx) = 1;
                    O(i(x),j(x),t(x)) = O(i(x),j(x),t(x)) + 1;
                end
            end
        end
    end

    % Brute-force maximization
    step = 0.01:0.01:0.99;
    index = 0;
    L_sum = zeros(length(step)^2,1);
    P_array(:,:,:) = zeros(numStates,numStates,length(step)^2);

    for P_12 = step
        P_11 = 1 - P_12;
        for P_22 = step
            P_21 = 1 - P_22;
            index = index + 1;
            P = [P_11 P_12;P_21 P_22];
            P_array(:,:,index) = P;
            for ii = 1:max(i)
                for jj = 1:max(j)
                    for tt = 1:max(t)
                        P_t = P^tt;
                        L = O(ii,jj,tt)*log(P_t(ii,jj));
                        L_sum(index) = L + L_sum(index);
                    end
                end
            end
        end
    end

    MLE = L_sum == max(L_sum);
    P = P_array(:,:,MLE);

end